import argparse
import sys
from pathlib import Path

from maildirclean.filedir import delete_all_from_selected_email

from .maildir import MailDir, TopSender, parse_maildir


class InputError(Exception):
    """Exception raised for custom error scenarios.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self):
        self.message = "Please enter a valid number or 'q' to quit"
        super().__init__(self.message)


def parse_arguments() -> argparse.Namespace:
    """Parse command line arguments

    Returns: Namespace object corresponding to parsed arguments

    """
    parser = argparse.ArgumentParser(
        description="Analyze email metadata from a maildir directory"
    )
    parser.add_argument(
        "maildir", type=str, help="Path to the maildir directory to analyze"
    )
    parser.add_argument(
        "--top",
        "-t",
        type=int,
        default=5,
        help="Number of top senders to display (default: 5)",
    )

    parser.add_argument(
        "--no-confirm",
        dest="confirm",
        action="store_false",
        default=True,
        help="Skip confirmation prompts",
    )

    parser.add_argument(
        "--verbose", "-v", action="store_true", help="Enable verbose output"
    )

    args = parser.parse_args()
    return args


def cli():
    args = parse_arguments()

    maildir_path = Path(args.maildir)
    if not maildir_path.exists() or not maildir_path.is_dir():
        print(f"Error: {args.maildir} is not a valid directory", file=sys.stderr)
        return 1

    if args.verbose:
        print(f"Analyzing emails in {maildir_path}...")

    run_loop(args, maildir_path)

    return 0


def run_loop(args: argparse.Namespace, maildir_path: str | Path):

    # Set up maildir
    maildir = parse_maildir(maildir_path)
    if args.verbose:
        print(f"Found {len(maildir._df)} emails")

    # Main running loop
    while True:
        top_senders = maildir.get_top_n_senders(args.top)

        if not top_senders:
            print("No senders found in the maildir", file=sys.stderr)
            return 0

        result = []
        for i, sender in enumerate(top_senders, 1):
            names_str = ", ".join(sender.names[:5])  # Limit to first 5 names
            if len(sender.names) > 5:
                names_str += f" and {len(sender.names) - 5} more"

            result.append(
                f"{i}. {sender.email} - Email count: {sender.count} - Names used: {names_str}"
            )

        output = "\n".join(
            [f"Top {len(top_senders)} senders in {maildir_path}:", "=" * 40, *result]
        )

        print(output)
        if not user_input_loop(top_senders, maildir, args.confirm):
            break


def user_input_loop(top_senders: list[TopSender], maildir: MailDir, confirm) -> bool:
    user_input = input("> ").strip()

    while True:
        try:
            return handle_user_input(user_input, top_senders, maildir, confirm)
        except InputError as ex:
            print(ex.message)
            user_input = input("> ").strip()


def parse_selections(user_input, max_selection):
    """
    Parse user input into a set of valid selections.

    Args:
        user_input (str): User input string with numbers, comma-separated lists, or ranges
        max_selection (int): Maximum allowed selection number

    Returns:
        set: Set of valid selection numbers

    Raises:
        ValueError: If any input is invalid or out of range
    """
    selections = set()

    # Clean up the input
    user_input = user_input.strip()

    # Split by comma
    items = [item.strip() for item in user_input.split(",")]

    for item in items:
        if "-" in item:
            # Handle range (e.g., "1-3" or "4 - 5")
            range_parts = [part.strip() for part in item.split("-")]
            if (
                len(range_parts) != 2
                or not range_parts[0].isdigit()
                or not range_parts[1].isdigit()
            ):
                raise ValueError(f"Invalid range format: {item}")

            start = int(range_parts[0])
            end = int(range_parts[1])

            if start > end:
                raise ValueError(
                    f"Invalid range: {item}. Start must be less than or equal to end."
                )

            selections.update(range(start, end + 1))
        elif item.isdigit():
            # Handle single number
            selections.add(int(item))
        else:
            raise ValueError(f"Invalid input: {item}")

    # Check if any selection is out of range
    out_of_range = [s for s in selections if s < 1 or s > max_selection]
    if out_of_range:
        raise ValueError(
            f"Selection(s) out of range: {', '.join(map(str, out_of_range))}. Valid range is 1-{max_selection}"
        )

    return selections


def handle_user_input(user_input, top_senders, maildir, confirm) -> bool:
    if user_input.lower() == "q":
        return False

    try:
        selections = parse_selections(user_input, len(top_senders))
        selected_senders = [top_senders[selection - 1] for selection in selections]
        delete_all_from_selected_email(selected_senders, maildir, confirm)
        return True
    except ValueError:
        raise InputError
